@@ -2,12 +2,13 @@ package main
 
 import (
 	"bytes"
+	"encoding/base64"
+	"errors"
 	"fmt"
-	"io"
-	"io/ioutil"
-	"log"
 	"os"
+	"runtime"
 	"sort"
+	"strings"
 	"time"
 
 	"git.sr.ht/~sircmpwn/getopt"
@@ -59,29 +60,30 @@ func getCommands(selected libui.Drawable) []*commands.Commands {
 }
 
 func execCommand(aerc *widgets.Aerc, ui *libui.UI, cmd []string) error {
-	cmds := getCommands(aerc.SelectedTab())
+	cmds := getCommands(aerc.SelectedTabContent())
 	for i, set := range cmds {
 		err := set.ExecuteCommand(aerc, cmd)
-		if _, ok := err.(commands.NoSuchCommand); ok {
-			if i == len(cmds)-1 {
-				return err
+		if err != nil {
+			if errors.As(err, new(commands.NoSuchCommand)) {
+				if i == len(cmds)-1 {
+					return err
+				}
+				continue
+			}
+			if errors.As(err, new(commands.ErrorExit)) {
+				ui.Exit()
+				return nil
 			}
-			continue
-		} else if _, ok := err.(commands.ErrorExit); ok {
-			ui.Exit()
-			return nil
-		} else if err != nil {
 			return err
-		} else {
-			break
 		}
+		break
 	}
 	return nil
 }
 
 func getCompletions(aerc *widgets.Aerc, cmd string) []string {
 	var completions []string
-	for _, set := range getCommands(aerc.SelectedTab()) {
+	for _, set := range getCommands(aerc.SelectedTabContent()) {
 		completions = append(completions, set.GetCompletions(aerc, cmd)...)
 	}
 	sort.Strings(completions)
@@ -90,21 +92,39 @@ func getCompletions(aerc *widgets.Aerc, cmd string) []string {
 
 // set at build time
 var Version string
+var Flags string
 
-func usage() {
-	log.Fatal("Usage: aerc [-v] [mailto:...]")
+func buildInfo() string {
+	info := Version
+	flags, _ := base64.StdEncoding.DecodeString(Flags)
+	if strings.Contains(string(flags), "notmuch") {
+		info += " +notmuch"
+	}
+	info += fmt.Sprintf(" (%s %s %s)",
+		runtime.Version(), runtime.GOARCH, runtime.GOOS)
+	return info
+}
+
+func usage(msg string) {
+	fmt.Fprintln(os.Stderr, msg)
+	fmt.Fprintln(os.Stderr, "usage: aerc [-v] [-a <account-name>] [mailto:...]")
+	os.Exit(1)
 }
 
 func setWindowTitle() {
+	logging.Debugf("Parsing terminfo")
 	ti, err := terminfo.LoadFromEnv()
 	if err != nil {
+		logging.Warnf("Cannot get terminfo: %v", err)
 		return
 	}
 
 	if !ti.Has(terminfo.HasStatusLine) {
+		logging.Infof("Terminal does not have status line support")
 		return
 	}
 
+	logging.Infof("Setting terminal title")
 	buf := new(bytes.Buffer)
 	ti.Fprintf(buf, terminfo.ToStatusLine)
 	fmt.Fprint(buf, "aerc")
@@ -114,23 +134,26 @@ func setWindowTitle() {
 
 func main() {
 	defer logging.PanicHandler()
-	opts, optind, err := getopt.Getopts(os.Args, "v")
+	opts, optind, err := getopt.Getopts(os.Args, "va:")
 	if err != nil {
-		log.Print(err)
-		usage()
+		usage("error: " + err.Error())
 		return
 	}
+	logging.BuildInfo = buildInfo()
+	var accts []string
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'v':
-			fmt.Println("aerc " + Version)
+		if opt.Option == 'v' {
+			fmt.Println("aerc " + logging.BuildInfo)
 			return
 		}
+		if opt.Option == 'a' {
+			accts = strings.Split(opt.Value, ",")
+		}
 	}
 	retryExec := false
 	args := os.Args[optind:]
 	if len(args) > 1 {
-		usage()
+		usage("error: invalid arguments")
 		return
 	} else if len(args) == 1 {
 		arg := args[0]
@@ -143,23 +166,15 @@ func main() {
 		retryExec = true
 	}
 
-	var (
-		logOut io.Writer
-		logger *log.Logger
-	)
 	if !isatty.IsTerminal(os.Stdout.Fd()) {
-		logOut = os.Stdout
-	} else {
-		logOut = ioutil.Discard
-		os.Stdout, _ = os.Open(os.DevNull)
+		logging.Init()
 	}
-	logger = log.New(logOut, "", log.LstdFlags)
-	logger.Println("Starting up aerc")
+	logging.Infof("Starting up version %s", logging.BuildInfo)
 
-	conf, err := config.LoadConfigFromFile(nil, logger)
+	conf, err := config.LoadConfigFromFile(nil, accts)
 	if err != nil {
 		fmt.Fprintf(os.Stderr, "Failed to load config: %v\n", err)
-		os.Exit(1)
+		os.Exit(1) //nolint:gocritic // PanicHandler does not need to run as it's not a panic
 	}
 
 	var (
@@ -170,10 +185,13 @@ func main() {
 	deferLoop := make(chan struct{})
 
 	c := crypto.New(conf.General.PgpProvider)
-	c.Init(logger)
+	err = c.Init()
+	if err != nil {
+		logging.Warnf("failed to initialise crypto interface: %v", err)
+	}
 	defer c.Close()
 
-	aerc = widgets.NewAerc(conf, logger, c, func(cmd []string) error {
+	aerc = widgets.NewAerc(conf, c, func(cmd []string) error {
 		return execCommand(aerc, ui, cmd)
 	}, func(cmd string) []string {
 		return getCompletions(aerc, cmd)
@@ -193,13 +211,13 @@ func main() {
 		ui.EnableMouse()
 	}
 
-	logger.Println("Starting Unix server")
-	as, err := lib.StartServer(logger)
+	as, err := lib.StartServer()
 	if err != nil {
-		logger.Printf("Failed to start Unix server: %v (non-fatal)", err)
+		logging.Warnf("Failed to start Unix server: %v", err)
 	} else {
 		defer as.Close()
 		as.OnMailto = aerc.Mailto
+		as.OnMbox = aerc.Mbox
 	}
 
 	// set the aerc version so that we can use it in the template funcs
@@ -211,7 +229,10 @@ func main() {
 		err := lib.ConnectAndExec(arg)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Failed to communicate to aerc: %v\n", err)
-			aerc.CloseBackends()
+			err = aerc.CloseBackends()
+			if err != nil {
+				logging.Warnf("failed to close backends: %v", err)
+			}
 			return
 		}
 	}
@@ -229,5 +250,8 @@ func main() {
 			time.Sleep(16 * time.Millisecond)
 		}
 	}
-	aerc.CloseBackends()
+	err = aerc.CloseBackends()
+	if err != nil {
+		logging.Warnf("failed to close backends: %v", err)
+	}
 }
