@@ -1,17 +1,20 @@
 package msg
 
 import (
+	"bufio"
 	"bytes"
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
+	"math/rand"
 	"os"
 	"path"
 	"strings"
+	"sync"
 
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -35,22 +38,29 @@ func (forward) Complete(aerc *widgets.Aerc, args []string) []string {
 }
 
 func (forward) Execute(aerc *widgets.Aerc, args []string) error {
-	opts, optind, err := getopt.Getopts(args, "AT:")
+	opts, optind, err := getopt.Getopts(args, "AFT:")
 	if err != nil {
 		return err
 	}
-	attach := false
+	attachAll := false
+	attachFull := false
 	template := ""
 	for _, opt := range opts {
 		switch opt.Option {
 		case 'A':
-			attach = true
+			attachAll = true
+		case 'F':
+			attachFull = true
 		case 'T':
 			template = opt.Value
 		}
 	}
 
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	if attachAll && attachFull {
+		return errors.New("Options -A and -F are mutually exclusive")
+	}
+
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	acct := widget.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
@@ -63,18 +73,21 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	acct.Logger().Println("Forwarding email " + msg.Envelope.MessageId)
+	logging.Infof("Forwarding email %s", msg.Envelope.MessageId)
 
 	h := &mail.Header{}
 	subject := "Fwd: " + msg.Envelope.Subject
 	h.SetSubject(subject)
 
-	if len(args) != 1 {
-		to := strings.Join(args[optind:], ", ")
-		tolist, err := mail.ParseAddressList(to)
+	var tolist []*mail.Address
+	to := strings.Join(args[optind:], ", ")
+	if strings.Contains(to, "@") {
+		tolist, err = mail.ParseAddressList(to)
 		if err != nil {
-			return fmt.Errorf("invalid to address(es): %v", err)
+			return fmt.Errorf("invalid to address(es): %w", err)
 		}
+	}
+	if len(tolist) > 0 {
 		h.SetAddressList("to", tolist)
 	}
 
@@ -94,7 +107,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 
 		tab := aerc.NewTab(composer, subject)
 		if !h.Has("to") {
-			composer.FocusRecipient()
+			composer.FocusEditor("to")
 		} else {
 			composer.FocusTerminal()
 		}
@@ -109,8 +122,8 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		return composer, nil
 	}
 
-	if attach {
-		tmpDir, err := ioutil.TempDir("", "aerc-tmp-attachment")
+	if attachFull {
+		tmpDir, err := os.MkdirTemp("", "aerc-tmp-attachment")
 		if err != nil {
 			return err
 		}
@@ -119,14 +132,20 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		store.FetchFull([]uint32{msg.Uid}, func(fm *types.FullMessage) {
 			tmpFile, err := os.Create(tmpFileName)
 			if err != nil {
-				println(err)
-				// TODO: Do something with the error
-				addTab()
+				logging.Warnf("failed to create temporary attachment: %v", err)
+				_, err = addTab()
+				if err != nil {
+					logging.Warnf("failed to add tab: %v", err)
+				}
 				return
 			}
 
 			defer tmpFile.Close()
-			io.Copy(tmpFile, fm.Content.Reader)
+			_, err = io.Copy(tmpFile, fm.Content.Reader)
+			if err != nil {
+				logging.Warnf("failed to write to tmpfile: %w", err)
+				return
+			}
 			composer, err := addTab()
 			if err != nil {
 				return
@@ -141,7 +160,6 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 			template = aerc.Config().Templates.Forwards
 		}
 
-		// TODO: add attachments!
 		part := lib.FindPlaintext(msg.BodyStructure, nil)
 		if part == nil {
 			part = lib.FindFirstNonMultipart(msg.BodyStructure, nil)
@@ -153,9 +171,43 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 		store.FetchBodyPart(msg.Uid, part, func(reader io.Reader) {
 			buf := new(bytes.Buffer)
-			buf.ReadFrom(reader)
+			scanner := bufio.NewScanner(reader)
+			for scanner.Scan() {
+				buf.WriteString(scanner.Text() + "\n")
+			}
 			original.Text = buf.String()
-			addTab()
+
+			// create composer
+			composer, err := addTab()
+			if err != nil {
+				return
+			}
+
+			// add attachments
+			if attachAll {
+				var mu sync.Mutex
+				parts := lib.FindAllNonMultipart(msg.BodyStructure, nil, nil)
+				for _, p := range parts {
+					if lib.EqualParts(p, part) {
+						continue
+					}
+					bs, err := msg.BodyStructure.PartAtIndex(p)
+					if err != nil {
+						logging.Errorf("cannot get PartAtIndex %v: %v", p, err)
+						continue
+					}
+					store.FetchBodyPart(msg.Uid, p, func(reader io.Reader) {
+						mime := fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
+						name, ok := bs.Params["name"]
+						if !ok {
+							name = fmt.Sprintf("%s_%s_%d", bs.MIMEType, bs.MIMESubType, rand.Uint64())
+						}
+						mu.Lock()
+						composer.AddPartAttachment(name, mime, bs.Params, reader)
+						mu.Unlock()
+					})
+				}
+			}
 		})
 	}
 	return nil
