@@ -34,12 +34,9 @@ func (Move) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	var (
-		createParents bool
-	)
+	var createParents bool
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'p':
+		if opt.Option == 'p' {
 			createParents = true
 		}
 	}
@@ -53,24 +50,22 @@ func (Move) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	acct, err := h.account()
-	if err != nil {
-		return err
-	}
 	_, isMsgView := h.msgProvider.(*widgets.MessageViewer)
 	if isMsgView {
 		aerc.RemoveTab(h.msgProvider)
 	}
-	store.Next()
-	acct.Messages().Invalidate()
+	marker := store.Marker()
+	marker.ClearVisualMark()
+	findNextNonDeleted(uids, store)
 	joinedArgs := strings.Join(args[optind:], " ")
 	store.Move(uids, joinedArgs, createParents, func(
-		msg types.WorkerMessage) {
-
+		msg types.WorkerMessage,
+	) {
 		switch msg := msg.(type) {
 		case *types.Done:
 			aerc.PushStatus("Message moved to "+joinedArgs, 10*time.Second)
 		case *types.Error:
+			marker.Remark()
 			aerc.PushError(msg.Error.Error())
 		}
 	})
