@@ -1,6 +1,8 @@
 package msgview
 
 import (
+	"errors"
+
 	"git.sr.ht/~rjarry/aerc/commands/account"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/widgets"
@@ -25,8 +27,11 @@ func (NextPrevMsg) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	mv, _ := aerc.SelectedTab().(*widgets.MessageViewer)
+	mv, _ := aerc.SelectedTabContent().(*widgets.MessageViewer)
 	acct := mv.SelectedAccount()
+	if acct == nil {
+		return errors.New("No account selected")
+	}
 	store := mv.Store()
 	err = account.ExecuteNextPrevMessage(args, acct, pct, n)
 	if err != nil {
@@ -37,7 +42,7 @@ func (NextPrevMsg) Execute(aerc *widgets.Aerc, args []string) error {
 		aerc.RemoveTab(mv)
 		return nil
 	}
-	lib.NewMessageStoreView(nextMsg, store, aerc.DecryptKeys,
+	lib.NewMessageStoreView(nextMsg, store, aerc.Crypto, aerc.DecryptKeys,
 		func(view lib.MessageView, err error) {
 			if err != nil {
 				aerc.PushError(err.Error())
