@@ -2,9 +2,12 @@ package lib
 
 import (
 	"io"
+	"sync"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -18,18 +21,13 @@ type MessageStore struct {
 
 	// Ordered list of known UIDs
 	uids    []uint32
-	Threads []*types.Thread
+	threads []*types.Thread
 
-	selected        int
-	reselect        *models.MessageInfo
-	bodyCallbacks   map[uint32][]func(*types.FullMessage)
-	headerCallbacks map[uint32][]func(*types.MessageInfo)
+	selectedUid   uint32
+	bodyCallbacks map[uint32][]func(*types.FullMessage)
 
-	//marking
-	marked         map[uint32]struct{}
-	lastMarked     map[uint32]struct{}
-	visualStartUid uint32
-	visualMarkMode bool
+	// marking
+	marker marker.Marker
 
 	// Search/filter results
 	results     []uint32
@@ -53,19 +51,23 @@ type MessageStore struct {
 	triggerNewEmail        func(*models.MessageInfo)
 	triggerDirectoryChange func()
 
-	dirInfoUpdateDebounce *time.Timer
-	dirInfoUpdateDelay    time.Duration
+	threadBuilderDebounce *time.Timer
+	threadBuilderDelay    time.Duration
+	threadCallback        func()
+
+	// threads mutex protects the store.threads and store.threadCallback
+	threadsMutex sync.Mutex
 }
 
+const MagicUid = 0xFFFFFFFF
+
 func NewMessageStore(worker *types.Worker,
 	dirInfo *models.DirectoryInfo,
 	defaultSortCriteria []*types.SortCriterion,
-	thread bool, clientThreads bool,
+	thread bool, clientThreads bool, clientThreadsDelay time.Duration,
 	triggerNewEmail func(*models.MessageInfo),
-	triggerDirectoryChange func()) *MessageStore {
-
-	dirInfoUpdateDelay := 5 * time.Second
-
+	triggerDirectoryChange func(),
+) *MessageStore {
 	if !dirInfo.Caps.Thread {
 		clientThreads = true
 	}
@@ -75,10 +77,9 @@ func NewMessageStore(worker *types.Worker,
 		DirInfo:  *dirInfo,
 		Messages: make(map[uint32]*models.MessageInfo),
 
-		selected:        0,
-		marked:          make(map[uint32]struct{}),
-		bodyCallbacks:   make(map[uint32][]func(*types.FullMessage)),
-		headerCallbacks: make(map[uint32][]func(*types.MessageInfo)),
+		selectedUid: MagicUid,
+
+		bodyCallbacks: make(map[uint32][]func(*types.FullMessage)),
 
 		threadedView: thread,
 		buildThreads: clientThreads,
@@ -93,14 +94,13 @@ func NewMessageStore(worker *types.Worker,
 		triggerNewEmail:        triggerNewEmail,
 		triggerDirectoryChange: triggerDirectoryChange,
 
-		dirInfoUpdateDelay:    dirInfoUpdateDelay,
-		dirInfoUpdateDebounce: time.NewTimer(dirInfoUpdateDelay),
+		threadBuilderDelay: clientThreadsDelay,
 	}
 }
 
 func (store *MessageStore) FetchHeaders(uids []uint32,
-	cb func(*types.MessageInfo)) {
-
+	cb func(types.WorkerMessage),
+) {
 	// TODO: this could be optimized by pre-allocating toFetch and trimming it
 	// at the end. In practice we expect to get most messages back in one frame.
 	var toFetch []uint32
@@ -108,24 +108,18 @@ func (store *MessageStore) FetchHeaders(uids []uint32,
 		if _, ok := store.pendingHeaders[uid]; !ok {
 			toFetch = append(toFetch, uid)
 			store.pendingHeaders[uid] = nil
-			if cb != nil {
-				if list, ok := store.headerCallbacks[uid]; ok {
-					store.headerCallbacks[uid] = append(list, cb)
-				} else {
-					store.headerCallbacks[uid] = []func(*types.MessageInfo){cb}
-				}
-			}
 		}
 	}
 	if len(toFetch) > 0 {
 		store.worker.PostAction(&types.FetchMessageHeaders{Uids: toFetch}, func(msg types.WorkerMessage) {
-			switch msg.(type) {
-			case *types.Error:
+			if _, ok := msg.(*types.Error); ok {
 				for _, uid := range toFetch {
 					delete(store.pendingHeaders, uid)
-					delete(store.headerCallbacks, uid)
 				}
 			}
+			if cb != nil {
+				cb(msg)
+			}
 		})
 	}
 }
@@ -151,8 +145,7 @@ func (store *MessageStore) FetchFull(uids []uint32, cb func(*types.FullMessage))
 		store.worker.PostAction(&types.FetchFullMessages{
 			Uids: toFetch,
 		}, func(msg types.WorkerMessage) {
-			switch msg.(type) {
-			case *types.Error:
+			if _, ok := msg.(*types.Error); ok {
 				for _, uid := range toFetch {
 					delete(store.pendingBodies, uid)
 					delete(store.bodyCallbacks, uid)
@@ -163,7 +156,6 @@ func (store *MessageStore) FetchFull(uids []uint32, cb func(*types.FullMessage))
 }
 
 func (store *MessageStore) FetchBodyPart(uid uint32, part []int, cb func(io.Reader)) {
-
 	store.worker.PostAction(&types.FetchMessageBodyPart{
 		Uid:  uid,
 		Part: part,
@@ -196,10 +188,8 @@ func merge(to *models.MessageInfo, from *models.MessageInfo) {
 
 func (store *MessageStore) Update(msg types.WorkerMessage) {
 	update := false
+	updateThreads := false
 	directoryChange := false
-	if store.reselect == nil {
-		store.SetReselect(store.Selected())
-	}
 	switch msg := msg.(type) {
 	case *types.DirectoryInfo:
 		store.DirInfo = *msg.Info
@@ -219,14 +209,15 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		store.Messages = newMap
 		store.uids = msg.Uids
-		store.checkMark()
-		update = true
+		if store.threadedView {
+			store.runThreadBuilderNow()
+		}
 	case *types.DirectoryThreaded:
 		var uids []uint32
 		newMap := make(map[uint32]*models.MessageInfo)
 
 		for i := len(msg.Threads) - 1; i >= 0; i-- {
-			msg.Threads[i].Walk(func(t *types.Thread, level int, currentErr error) error {
+			_ = msg.Threads[i].Walk(func(t *types.Thread, level int, currentErr error) error {
 				uid := t.Uid
 				uids = append([]uint32{uid}, uids...)
 				if msg, ok := store.Messages[uid]; ok {
@@ -240,16 +231,13 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		store.Messages = newMap
 		store.uids = uids
-		store.checkMark()
-		store.Threads = msg.Threads
+		store.threads = msg.Threads
 		update = true
 	case *types.MessageInfo:
 		if existing, ok := store.Messages[msg.Info.Uid]; ok && existing != nil {
 			merge(existing, msg.Info)
-		} else {
-			if msg.Info.Envelope != nil {
-				store.Messages[msg.Info.Uid] = msg.Info
-			}
+		} else if msg.Info.Envelope != nil {
+			store.Messages[msg.Info.Uid] = msg.Info
 		}
 		seen := false
 		recent := false
@@ -265,16 +253,12 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		if _, ok := store.pendingHeaders[msg.Info.Uid]; msg.Info.Envelope != nil && ok {
 			delete(store.pendingHeaders, msg.Info.Uid)
-			if cbs, ok := store.headerCallbacks[msg.Info.Uid]; ok {
-				for _, cb := range cbs {
-					cb(msg)
-				}
-			}
 		}
 		if store.builder != nil {
 			store.builder.Update(msg.Info)
 		}
 		update = true
+		updateThreads = true
 	case *types.FullMessage:
 		if _, ok := store.pendingBodies[msg.Content.Uid]; ok {
 			delete(store.pendingBodies, msg.Content.Uid)
@@ -296,7 +280,6 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			toDelete[uid] = nil
 			delete(store.Messages, uid)
 			delete(store.Deleted, uid)
-			delete(store.marked, uid)
 		}
 		uids := make([]uint32, len(store.uids)-len(msg.Uids))
 		j := 0
@@ -316,8 +299,8 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 		store.results = newResults
 
-		for _, thread := range store.Threads {
-			thread.Walk(func(t *types.Thread, _ int, _ error) error {
+		for _, thread := range store.Threads() {
+			_ = thread.Walk(func(t *types.Thread, _ int, _ error) error {
 				if _, deleted := toDelete[t.Uid]; deleted {
 					t.Deleted = true
 				}
@@ -326,10 +309,11 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		}
 
 		update = true
+		updateThreads = true
 	}
 
 	if update {
-		store.update()
+		store.update(updateThreads)
 	}
 
 	if directoryChange && store.triggerDirectoryChange != nil {
@@ -349,14 +333,14 @@ func (store *MessageStore) OnUpdateDirs(fn func()) {
 	store.onUpdateDirs = fn
 }
 
-func (store *MessageStore) update() {
+func (store *MessageStore) update(threads bool) {
 	if store.onUpdate != nil {
 		store.onUpdate(store)
 	}
 	if store.onUpdateDirs != nil {
 		store.onUpdateDirs()
 	}
-	if store.BuildThreads() {
+	if store.BuildThreads() && store.ThreadedView() && threads {
 		store.runThreadBuilder()
 	}
 }
@@ -366,13 +350,20 @@ func (store *MessageStore) SetThreadedView(thread bool) {
 	if store.buildThreads {
 		if store.threadedView {
 			store.runThreadBuilder()
+		} else if store.threadBuilderDebounce != nil {
+			store.threadBuilderDebounce.Stop()
 		}
-		store.Reselect()
 		return
 	}
 	store.Sort(store.sortCriteria, nil)
 }
 
+func (store *MessageStore) Threads() []*types.Thread {
+	store.threadsMutex.Lock()
+	defer store.threadsMutex.Unlock()
+	return store.threads
+}
+
 func (store *MessageStore) ThreadedView() bool {
 	return store.threadedView
 }
@@ -382,26 +373,77 @@ func (store *MessageStore) BuildThreads() bool {
 }
 
 func (store *MessageStore) runThreadBuilder() {
+	if store.threadBuilderDebounce != nil {
+		if store.threadBuilderDebounce.Stop() {
+			logging.Infof("thread builder debounced")
+		}
+	}
+	store.threadBuilderDebounce = time.AfterFunc(store.threadBuilderDelay, func() {
+		store.runThreadBuilderNow()
+	})
+}
+
+// runThreadBuilderNow runs the threadbuilder without any debounce logic
+func (store *MessageStore) runThreadBuilderNow() {
 	if store.builder == nil {
-		store.builder = NewThreadBuilder(store.worker.Logger)
+		store.builder = NewThreadBuilder()
 		for _, msg := range store.Messages {
 			store.builder.Update(msg)
 		}
 	}
-	store.Threads = store.builder.Threads(store.uids)
+	// build new threads
+	th := store.builder.Threads(store.uids)
+
+	// save local threads to the message store variable and
+	// run callback if defined (callback should reposition cursor)
+	store.threadsMutex.Lock()
+	store.threads = th
+	if store.threadCallback != nil {
+		store.threadCallback()
+	}
+	store.threadsMutex.Unlock()
+
+	// invalidate message list
+	if store.onUpdate != nil {
+		store.onUpdate(store)
+	}
+}
+
+// SelectedThread returns the thread with the UID from the selected message
+func (store *MessageStore) SelectedThread() *types.Thread {
+	var thread *types.Thread
+	for _, root := range store.Threads() {
+		found := false
+		err := root.Walk(func(t *types.Thread, _ int, _ error) error {
+			if t.Uid == store.SelectedUid() {
+				thread = t
+				found = true
+			}
+			return nil
+		})
+		if err != nil {
+			logging.Errorf("SelectedThread failed: %w", err)
+		}
+		if found {
+			break
+		}
+	}
+	return thread
 }
 
 func (store *MessageStore) Delete(uids []uint32,
-	cb func(msg types.WorkerMessage)) {
-
+	cb func(msg types.WorkerMessage),
+) {
 	for _, uid := range uids {
 		store.Deleted[uid] = nil
 	}
 
 	store.worker.PostAction(&types.DeleteMessages{Uids: uids},
 		func(msg types.WorkerMessage) {
-			switch msg.(type) {
-			case *types.Error:
+			if _, ok := msg.(*types.Error); ok {
+				store.revertDeleted(uids)
+			}
+			if _, ok := msg.(*types.Unsupported); ok {
 				store.revertDeleted(uids)
 			}
 			cb(msg)
@@ -410,15 +452,13 @@ func (store *MessageStore) Delete(uids []uint32,
 
 func (store *MessageStore) revertDeleted(uids []uint32) {
 	for _, uid := range uids {
-		if _, ok := store.Deleted[uid]; ok {
-			delete(store.Deleted, uid)
-		}
+		delete(store.Deleted, uid)
 	}
 }
 
 func (store *MessageStore) Copy(uids []uint32, dest string, createDest bool,
-	cb func(msg types.WorkerMessage)) {
-
+	cb func(msg types.WorkerMessage),
+) {
 	if createDest {
 		store.worker.PostAction(&types.CreateDirectory{
 			Directory: dest,
@@ -433,8 +473,8 @@ func (store *MessageStore) Copy(uids []uint32, dest string, createDest bool,
 }
 
 func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
-	cb func(msg types.WorkerMessage)) {
-
+	cb func(msg types.WorkerMessage),
+) {
 	for _, uid := range uids {
 		store.Deleted[uid] = nil
 	}
@@ -446,7 +486,7 @@ func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
 		}, nil) // quiet doesn't return an error, don't want the done cb here
 	}
 
-	store.worker.PostAction(&types.CopyMessages{
+	store.worker.PostAction(&types.MoveMessages{
 		Destination: dest,
 		Uids:        uids,
 	}, func(msg types.WorkerMessage) {
@@ -455,14 +495,14 @@ func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
 			store.revertDeleted(uids)
 			cb(msg)
 		case *types.Done:
-			store.Delete(uids, cb)
+			cb(msg)
 		}
 	})
 }
 
 func (store *MessageStore) Flag(uids []uint32, flag models.Flag,
-	enable bool, cb func(msg types.WorkerMessage)) {
-
+	enable bool, cb func(msg types.WorkerMessage),
+) {
 	store.worker.PostAction(&types.FlagMessages{
 		Enable: enable,
 		Flag:   flag,
@@ -471,8 +511,8 @@ func (store *MessageStore) Flag(uids []uint32, flag models.Flag,
 }
 
 func (store *MessageStore) Answered(uids []uint32, answered bool,
-	cb func(msg types.WorkerMessage)) {
-
+	cb func(msg types.WorkerMessage),
+) {
 	store.worker.PostAction(&types.AnsweredMessages{
 		Answered: answered,
 		Uids:     uids,
@@ -480,7 +520,6 @@ func (store *MessageStore) Answered(uids []uint32, answered bool,
 }
 
 func (store *MessageStore) Uids() []uint32 {
-
 	if store.ThreadedView() && store.builder != nil {
 		if uids := store.builder.Uids(); len(uids) > 0 {
 			return uids
@@ -490,177 +529,27 @@ func (store *MessageStore) Uids() []uint32 {
 }
 
 func (store *MessageStore) Selected() *models.MessageInfo {
-	uids := store.Uids()
-	idx := len(uids) - store.selected - 1
-	if len(uids) == 0 || idx < 0 || idx >= len(uids) {
-		return nil
-	}
-	return store.Messages[uids[idx]]
+	return store.Messages[store.selectedUid]
 }
 
-func (store *MessageStore) SelectedIndex() int {
-	return store.selected
+func (store *MessageStore) SelectedUid() uint32 {
+	if store.selectedUid == MagicUid && len(store.Uids()) > 0 {
+		uids := store.Uids()
+		store.selectedUid = uids[len(uids)-1]
+	}
+	return store.selectedUid
 }
 
-func (store *MessageStore) Select(index int) {
-	uids := store.Uids()
-	store.selected = index
-	if store.selected < 0 {
-		store.selected = len(uids) - 1
-	} else if store.selected > len(uids) {
-		store.selected = len(uids)
+func (store *MessageStore) Select(uid uint32) {
+	store.threadsMutex.Lock()
+	if store.threadCallback != nil {
+		store.threadCallback = nil
 	}
-	store.updateVisual()
-}
-
-func (store *MessageStore) Reselect() {
-	if store.reselect == nil {
-		return
+	store.threadsMutex.Unlock()
+	store.selectedUid = uid
+	if store.marker != nil {
+		store.marker.UpdateVisualMark()
 	}
-	uid := store.reselect.Uid
-	newIdx := 0
-	for idx, uidStore := range store.Uids() {
-		if uidStore == uid {
-			newIdx = len(store.Uids()) - idx - 1
-			break
-		}
-	}
-	store.reselect = nil
-	store.Select(newIdx)
-}
-
-func (store *MessageStore) SetReselect(info *models.MessageInfo) {
-	store.reselect = info
-}
-
-// Mark sets the marked state on a MessageInfo
-func (store *MessageStore) Mark(uid uint32) {
-	if store.visualMarkMode {
-		// visual mode has override, bogus input from user
-		return
-	}
-	store.marked[uid] = struct{}{}
-}
-
-// Unmark removes the marked state on a MessageInfo
-func (store *MessageStore) Unmark(uid uint32) {
-	if store.visualMarkMode {
-		// user probably wanted to clear the visual marking
-		store.ClearVisualMark()
-		return
-	}
-	delete(store.marked, uid)
-}
-
-func (store *MessageStore) Remark() {
-	store.marked = store.lastMarked
-}
-
-// ToggleMark toggles the marked state on a MessageInfo
-func (store *MessageStore) ToggleMark(uid uint32) {
-	if store.visualMarkMode {
-		// visual mode has override, bogus input from user
-		return
-	}
-	if store.IsMarked(uid) {
-		store.Unmark(uid)
-	} else {
-		store.Mark(uid)
-	}
-}
-
-// resetMark removes the marking from all messages
-func (store *MessageStore) resetMark() {
-	store.lastMarked = store.marked
-	store.marked = make(map[uint32]struct{})
-}
-
-// checkMark checks that no stale uids remain marked
-func (store *MessageStore) checkMark() {
-	for mark := range store.marked {
-		present := false
-		for _, uid := range store.uids {
-			if mark == uid {
-				present = true
-				break
-			}
-		}
-		if !present {
-			delete(store.marked, mark)
-		}
-	}
-}
-
-//IsMarked checks whether a MessageInfo has been marked
-func (store *MessageStore) IsMarked(uid uint32) bool {
-	_, marked := store.marked[uid]
-	return marked
-}
-
-//ToggleVisualMark enters or leaves the visual marking mode
-func (store *MessageStore) ToggleVisualMark() {
-	store.visualMarkMode = !store.visualMarkMode
-	switch store.visualMarkMode {
-	case true:
-		// just entered visual mode, reset whatever marking was already done
-		store.resetMark()
-		store.visualStartUid = store.Selected().Uid
-		store.marked[store.visualStartUid] = struct{}{}
-	case false:
-		// visual mode ended, nothing to do
-		return
-	}
-}
-
-//ClearVisualMark leaves the visual marking mode and resets any marking
-func (store *MessageStore) ClearVisualMark() {
-	store.resetMark()
-	store.visualMarkMode = false
-	store.visualStartUid = 0
-}
-
-// Marked returns the uids of all marked messages
-func (store *MessageStore) Marked() []uint32 {
-	marked := make([]uint32, len(store.marked))
-	i := 0
-	for uid := range store.marked {
-		marked[i] = uid
-		i++
-	}
-	return marked
-}
-
-func (store *MessageStore) updateVisual() {
-	if !store.visualMarkMode {
-		// nothing to do
-		return
-	}
-	startIdx := store.visualStartIdx()
-	if startIdx < 0 {
-		// something deleted the startuid, abort the marking process
-		store.ClearVisualMark()
-		return
-	}
-	uidLen := len(store.Uids())
-	// store.selected is the inverted form of the actual array
-	selectedIdx := uidLen - store.selected - 1
-	var visUids []uint32
-	if selectedIdx > startIdx {
-		visUids = store.Uids()[startIdx : selectedIdx+1]
-	} else {
-		visUids = store.Uids()[selectedIdx : startIdx+1]
-	}
-	store.resetMark()
-	for _, uid := range visUids {
-		store.marked[uid] = struct{}{}
-	}
-	missing := make([]uint32, 0)
-	for _, uid := range visUids {
-		if msg := store.Messages[uid]; msg == nil {
-			missing = append(missing, uid)
-		}
-	}
-	store.FetchHeaders(missing, nil)
 }
 
 func (store *MessageStore) NextPrev(delta int) {
@@ -668,21 +557,44 @@ func (store *MessageStore) NextPrev(delta int) {
 	if len(uids) == 0 {
 		return
 	}
-	store.selected += delta
-	if store.selected < 0 {
-		store.selected = 0
+
+	uid := store.SelectedUid()
+
+	newIdx := store.FindIndexByUid(uid)
+	if newIdx < 0 {
+		store.Select(uids[len(uids)-1])
+		return
 	}
-	if store.selected >= len(uids) {
-		store.selected = len(uids) - 1
+
+	newIdx -= delta
+	if newIdx >= len(uids) {
+		newIdx = len(uids) - 1
+	} else if newIdx < 0 {
+		newIdx = 0
 	}
-	store.updateVisual()
+
+	store.Select(uids[newIdx])
+
+	if store.BuildThreads() && store.ThreadedView() {
+		store.threadsMutex.Lock()
+		store.threadCallback = func() {
+			if uids := store.Uids(); len(uids) > newIdx {
+				store.selectedUid = uids[newIdx]
+			}
+		}
+		store.threadsMutex.Unlock()
+	}
+
+	if store.marker != nil {
+		store.marker.UpdateVisualMark()
+	}
+
 	nextResultIndex := len(store.results) - store.resultIndex - 2*delta
 	if nextResultIndex < 0 || nextResultIndex >= len(store.results) {
 		return
 	}
 	nextResultUid := store.results[nextResultIndex]
-	selectedUid := uids[len(uids)-store.selected-1]
-	if nextResultUid == selectedUid {
+	if nextResultUid == store.SelectedUid() {
 		store.resultIndex += delta
 	}
 }
@@ -699,8 +611,7 @@ func (store *MessageStore) Search(args []string, cb func([]uint32)) {
 	store.worker.PostAction(&types.SearchDirectory{
 		Argv: args,
 	}, func(msg types.WorkerMessage) {
-		switch msg := msg.(type) {
-		case *types.SearchResults:
+		if msg, ok := msg.(*types.SearchResults); ok {
 			allowedUids := store.Uids()
 			uids := make([]uint32, 0, len(msg.Uids))
 			for _, uid := range msg.Uids {
@@ -728,21 +639,12 @@ func (store *MessageStore) SetFilter(args []string) {
 }
 
 func (store *MessageStore) ApplyClear() {
-	if store.reselect == nil {
-		store.SetReselect(store.Selected())
-	}
 	store.filter = []string{"filter"}
 	store.results = nil
 	if store.onFilterChange != nil {
 		store.onFilterChange(store)
 	}
-	cb := func(msg types.WorkerMessage) {
-		switch msg.(type) {
-		case *types.Done:
-			store.Reselect()
-		}
-	}
-	store.Sort(nil, cb)
+	store.Sort(nil, nil)
 }
 
 func (store *MessageStore) nextPrevResult(delta int) {
@@ -756,14 +658,8 @@ func (store *MessageStore) nextPrevResult(delta int) {
 	if store.resultIndex < 0 {
 		store.resultIndex = len(store.results) - 1
 	}
-	uids := store.Uids()
-	for i, uid := range uids {
-		if store.results[len(store.results)-store.resultIndex-1] == uid {
-			store.Select(len(uids) - i - 1)
-			break
-		}
-	}
-	store.update()
+	store.Select(store.results[len(store.results)-store.resultIndex-1])
+	store.update(false)
 }
 
 func (store *MessageStore) NextResult() {
@@ -775,7 +671,8 @@ func (store *MessageStore) PrevResult() {
 }
 
 func (store *MessageStore) ModifyLabels(uids []uint32, add, remove []string,
-	cb func(msg types.WorkerMessage)) {
+	cb func(msg types.WorkerMessage),
+) {
 	store.worker.PostAction(&types.ModifyLabels{
 		Uids:   uids,
 		Add:    add,
@@ -784,14 +681,11 @@ func (store *MessageStore) ModifyLabels(uids []uint32, add, remove []string,
 }
 
 func (store *MessageStore) Sort(criteria []*types.SortCriterion, cb func(types.WorkerMessage)) {
-	if criteria == nil {
-		criteria = store.sortCriteria
-	} else {
-		store.sortCriteria = criteria
-	}
+	store.sortCriteria = criteria
 	store.Sorting = true
 
 	handle_return := func(msg types.WorkerMessage) {
+		store.Select(store.SelectedUid())
 		store.Sorting = false
 		if cb != nil {
 			cb(msg)
@@ -811,12 +705,38 @@ func (store *MessageStore) Sort(criteria []*types.SortCriterion, cb func(types.W
 	}
 }
 
-// returns the index of needle in haystack or -1 if not found
-func (store *MessageStore) visualStartIdx() int {
+func (store *MessageStore) GetCurrentSortCriteria() []*types.SortCriterion {
+	return store.sortCriteria
+}
+
+func (store *MessageStore) SetMarker(m marker.Marker) {
+	store.marker = m
+}
+
+func (store *MessageStore) Marker() marker.Marker {
+	if store.marker == nil {
+		store.marker = marker.New(store)
+	}
+	return store.marker
+}
+
+// FindIndexByUid returns the index in store.Uids() or -1 if not found
+func (store *MessageStore) FindIndexByUid(uid uint32) int {
 	for idx, u := range store.Uids() {
-		if u == store.visualStartUid {
+		if u == uid {
 			return idx
 		}
 	}
 	return -1
 }
+
+// Capabilities returns a models.Capabilities struct or nil if not available
+func (store *MessageStore) Capabilities() *models.Capabilities {
+	return store.DirInfo.Caps
+}
+
+// SelectedIndex returns the index of the selected message in the uid list or
+// -1 if not found
+func (store *MessageStore) SelectedIndex() int {
+	return store.FindIndexByUid(store.selectedUid)
+}
