@@ -4,8 +4,8 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"log"
 	"net/url"
+	"sort"
 	"strings"
 	"time"
 
@@ -16,7 +16,9 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 )
 
@@ -28,7 +30,6 @@ type Aerc struct {
 	conf        *config.AercConfig
 	focused     ui.Interactive
 	grid        *ui.Grid
-	logger      *log.Logger
 	simulating  int
 	statusbar   *ui.Stack
 	statusline  *StatusLine
@@ -38,6 +39,8 @@ type Aerc struct {
 	ui          *ui.UI
 	beep        func() error
 	dialog      ui.DrawableInteractive
+
+	Crypto crypto.Provider
 }
 
 type Choice struct {
@@ -46,10 +49,11 @@ type Choice struct {
 	Command []string
 }
 
-func NewAerc(conf *config.AercConfig, logger *log.Logger,
-	cmd func(cmd []string) error, complete func(cmd string) []string,
-	cmdHistory lib.History) *Aerc {
-
+func NewAerc(conf *config.AercConfig,
+	crypto crypto.Provider, cmd func(cmd []string) error,
+	complete func(cmd string) []string, cmdHistory lib.History,
+	deferLoop chan struct{},
+) *Aerc {
 	tabs := ui.NewTabs(&conf.Ui)
 
 	statusbar := ui.NewStack(conf.Ui)
@@ -57,11 +61,11 @@ func NewAerc(conf *config.AercConfig, logger *log.Logger,
 	statusbar.Push(statusline)
 
 	grid := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_EXACT, ui.Const(1)},
-		{ui.SIZE_WEIGHT, ui.Const(1)},
-		{ui.SIZE_EXACT, ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	grid.AddChild(tabs.TabStrip)
 	grid.AddChild(tabs.TabContent).At(1, 0)
@@ -74,23 +78,24 @@ func NewAerc(conf *config.AercConfig, logger *log.Logger,
 		cmdHistory: cmdHistory,
 		complete:   complete,
 		grid:       grid,
-		logger:     logger,
 		statusbar:  statusbar,
 		statusline: statusline,
 		prompts:    ui.NewStack(conf.Ui),
 		tabs:       tabs,
+		Crypto:     crypto,
 	}
 
 	statusline.SetAerc(aerc)
 	conf.Triggers.ExecuteCommand = cmd
 
 	for i, acct := range conf.Accounts {
-		view, err := NewAccountView(aerc, conf, &conf.Accounts[i], logger, aerc)
+		view, err := NewAccountView(aerc, conf, &conf.Accounts[i], aerc, deferLoop)
 		if err != nil {
-			tabs.Add(errorScreen(err.Error(), conf.Ui), acct.Name)
+			tabs.Add(errorScreen(err.Error(), conf.Ui), acct.Name, nil)
 		} else {
 			aerc.accounts[acct.Name] = view
-			tabs.Add(view, acct.Name)
+			conf := view.UiConfig()
+			tabs.Add(view, acct.Name, conf)
 		}
 	}
 
@@ -100,8 +105,14 @@ func NewAerc(conf *config.AercConfig, logger *log.Logger,
 		aerc.NewTab(wizard, "New account")
 	}
 
+	tabs.Select(0)
+
 	tabs.CloseTab = func(index int) {
-		switch content := aerc.tabs.Tabs[index].Content.(type) {
+		tab := aerc.tabs.Get(index)
+		if tab == nil {
+			return
+		}
+		switch content := tab.Content.(type) {
 		case *AccountView:
 			return
 		case *AccountWizard:
@@ -125,11 +136,11 @@ func (aerc *Aerc) OnBeep(f func() error) {
 
 func (aerc *Aerc) Beep() {
 	if aerc.beep == nil {
-		aerc.logger.Printf("should beep, but no beeper")
+		logging.Warnf("should beep, but no beeper")
 		return
 	}
 	if err := aerc.beep(); err != nil {
-		aerc.logger.Printf("tried to beep, but could not: %v", err)
+		logging.Errorf("tried to beep, but could not: %v", err)
 	}
 }
 
@@ -176,19 +187,62 @@ func (aerc *Aerc) Focus(focus bool) {
 func (aerc *Aerc) Draw(ctx *ui.Context) {
 	aerc.grid.Draw(ctx)
 	if aerc.dialog != nil {
-		aerc.dialog.Draw(ctx.Subcontext(4, ctx.Height()/2-2,
-			ctx.Width()-8, 4))
+		if w, h := ctx.Width(), ctx.Height(); w > 8 && h > 4 {
+			if d, ok := aerc.dialog.(Dialog); ok {
+				start, height := d.ContextHeight()
+				aerc.dialog.Draw(
+					ctx.Subcontext(4, start(h),
+						w-8, height(h)))
+			} else {
+				aerc.dialog.Draw(ctx.Subcontext(4, h/2-2, w-8, 4))
+			}
+		}
 	}
 }
 
+func (aerc *Aerc) HumanReadableBindings() []string {
+	var result []string
+	binds := aerc.getBindings()
+	format := func(s string) string {
+		s = strings.ReplaceAll(s, "<space>", " ")
+		return strings.ReplaceAll(s, "%", "%%")
+	}
+	fmtStr := "%10s %s"
+	for _, bind := range binds.Bindings {
+		result = append(result, fmt.Sprintf(fmtStr,
+			format(config.FormatKeyStrokes(bind.Input)),
+			format(config.FormatKeyStrokes(bind.Output)),
+		))
+	}
+	if binds.Globals && aerc.conf.Bindings.Global != nil {
+		for _, bind := range aerc.conf.Bindings.Global.Bindings {
+			result = append(result, fmt.Sprintf(fmtStr+" (Globals)",
+				format(config.FormatKeyStrokes(bind.Input)),
+				format(config.FormatKeyStrokes(bind.Output)),
+			))
+		}
+	}
+	result = append(result, fmt.Sprintf(fmtStr,
+		"$ex",
+		fmt.Sprintf("'%c'", binds.ExKey.Rune),
+	))
+	result = append(result, fmt.Sprintf(fmtStr,
+		"Globals",
+		fmt.Sprintf("%v", binds.Globals),
+	))
+	sort.Strings(result)
+	return result
+}
+
 func (aerc *Aerc) getBindings() *config.KeyBindings {
 	selectedAccountName := ""
 	if aerc.SelectedAccount() != nil {
 		selectedAccountName = aerc.SelectedAccount().acct.Name
 	}
-	switch view := aerc.SelectedTab().(type) {
+	switch view := aerc.SelectedTabContent().(type) {
 	case *AccountView:
-		return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageList, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "messages")
+		binds := aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageList, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "messages")
+		return aerc.conf.MergeContextualBinds(binds, config.BIND_CONTEXT_FOLDER, view.SelectedDirectory(), "messages")
 	case *AccountWizard:
 		return aerc.conf.Bindings.AccountWizard
 	case *Composer:
@@ -201,7 +255,12 @@ func (aerc *Aerc) getBindings() *config.KeyBindings {
 			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.Compose, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "compose")
 		}
 	case *MessageViewer:
-		return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageView, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "view")
+		switch view.Bindings() {
+		case "view::passthrough":
+			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageViewPassthrough, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "view::passthrough")
+		default:
+			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageView, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "view")
+		}
 	case *Terminal:
 		return aerc.conf.Bindings.Terminal
 	default:
@@ -272,7 +331,7 @@ func (aerc *Aerc) Event(event tcell.Event) bool {
 				aerc.BeginExCommand("")
 				return true
 			}
-			interactive, ok := aerc.tabs.Tabs[aerc.tabs.Selected].Content.(ui.Interactive)
+			interactive, ok := aerc.SelectedTabContent().(ui.Interactive)
 			if ok {
 				return interactive.Event(event)
 			}
@@ -293,12 +352,12 @@ func (aerc *Aerc) Config() *config.AercConfig {
 	return aerc.conf
 }
 
-func (aerc *Aerc) Logger() *log.Logger {
-	return aerc.logger
+func (aerc *Aerc) SelectedAccount() *AccountView {
+	return aerc.account(aerc.SelectedTabContent())
 }
 
-func (aerc *Aerc) SelectedAccount() *AccountView {
-	switch tab := aerc.SelectedTab().(type) {
+func (aerc *Aerc) account(d ui.Drawable) *AccountView {
+	switch tab := d.(type) {
 	case *AccountView:
 		return tab
 	case *MessageViewer:
@@ -309,34 +368,48 @@ func (aerc *Aerc) SelectedAccount() *AccountView {
 	return nil
 }
 
-func (aerc *Aerc) SelectedTab() ui.Drawable {
-	return aerc.tabs.Tabs[aerc.tabs.Selected].Content
+func (aerc *Aerc) SelectedAccountUiConfig() *config.UIConfig {
+	acct := aerc.SelectedAccount()
+	if acct == nil {
+		return &aerc.conf.Ui
+	}
+	return acct.UiConfig()
 }
 
-func (aerc *Aerc) SelectedTabIndex() int {
-	return aerc.tabs.Selected
+func (aerc *Aerc) SelectedTabContent() ui.Drawable {
+	tab := aerc.tabs.Selected()
+	if tab == nil {
+		return nil
+	}
+	return tab.Content
 }
 
-func (aerc *Aerc) NumTabs() int {
-	return len(aerc.tabs.Tabs)
+func (aerc *Aerc) SelectedTab() *ui.Tab {
+	return aerc.tabs.Selected()
 }
 
 func (aerc *Aerc) NewTab(clickable ui.Drawable, name string) *ui.Tab {
-	tab := aerc.tabs.Add(clickable, name)
-	aerc.tabs.Select(len(aerc.tabs.Tabs) - 1)
+	var uiConf *config.UIConfig = nil
+	if acct := aerc.account(clickable); acct != nil {
+		conf := acct.UiConfig()
+		uiConf = conf
+	}
+	tab := aerc.tabs.Add(clickable, name, uiConf)
+	aerc.UpdateStatus()
 	return tab
 }
 
 func (aerc *Aerc) RemoveTab(tab ui.Drawable) {
 	aerc.tabs.Remove(tab)
+	aerc.UpdateStatus()
 }
 
 func (aerc *Aerc) ReplaceTab(tabSrc ui.Drawable, tabTarget ui.Drawable, name string) {
 	aerc.tabs.Replace(tabSrc, tabTarget, name)
 }
 
-func (aerc *Aerc) MoveTab(i int) {
-	aerc.tabs.MoveTab(i)
+func (aerc *Aerc) MoveTab(i int, relative bool) {
+	aerc.tabs.MoveTab(i, relative)
 }
 
 func (aerc *Aerc) PinTab() {
@@ -356,42 +429,45 @@ func (aerc *Aerc) PrevTab() {
 }
 
 func (aerc *Aerc) SelectTab(name string) bool {
-	for i, tab := range aerc.tabs.Tabs {
-		if tab.Name == name {
-			aerc.tabs.Select(i)
-			return true
-		}
+	ok := aerc.tabs.SelectName(name)
+	if ok {
+		aerc.UpdateStatus()
 	}
-	return false
+	return ok
 }
 
 func (aerc *Aerc) SelectTabIndex(index int) bool {
-	for i := range aerc.tabs.Tabs {
-		if i == index {
-			aerc.tabs.Select(i)
-			return true
-		}
+	ok := aerc.tabs.Select(index)
+	if ok {
+		aerc.UpdateStatus()
 	}
-	return false
+	return ok
 }
 
 func (aerc *Aerc) TabNames() []string {
-	var names []string
-	for _, tab := range aerc.tabs.Tabs {
-		names = append(names, tab.Name)
-	}
-	return names
+	return aerc.tabs.Names()
 }
 
 func (aerc *Aerc) SelectPreviousTab() bool {
 	return aerc.tabs.SelectPrevious()
 }
 
-// TODO: Use per-account status lines, but a global ex line
 func (aerc *Aerc) SetStatus(status string) *StatusMessage {
 	return aerc.statusline.Set(status)
 }
 
+func (aerc *Aerc) UpdateStatus() {
+	if acct := aerc.SelectedAccount(); acct != nil {
+		acct.UpdateStatus()
+	} else {
+		aerc.ClearStatus()
+	}
+}
+
+func (aerc *Aerc) ClearStatus() {
+	aerc.statusline.Set("")
+}
+
 func (aerc *Aerc) SetError(status string) *StatusMessage {
 	return aerc.statusline.SetError(status)
 }
@@ -416,16 +492,14 @@ func (aerc *Aerc) focus(item ui.Interactive) {
 		aerc.focused.Focus(false)
 	}
 	aerc.focused = item
-	interactive, ok := aerc.tabs.Tabs[aerc.tabs.Selected].Content.(ui.Interactive)
+	interactive, ok := aerc.SelectedTabContent().(ui.Interactive)
 	if item != nil {
 		item.Focus(true)
 		if ok {
 			interactive.Focus(false)
 		}
-	} else {
-		if ok {
-			interactive.Focus(true)
-		}
+	} else if ok {
+		interactive.Focus(true)
 	}
 }
 
@@ -498,20 +572,27 @@ func (aerc *Aerc) RegisterChoices(choices []Choice) {
 }
 
 func (aerc *Aerc) Mailto(addr *url.URL) error {
-	acct := aerc.SelectedAccount()
-	if acct == nil {
-		return errors.New("No account selected")
-	}
-
 	var subject string
+	var body string
+	var acctName string
 	h := &mail.Header{}
 	to, err := mail.ParseAddressList(addr.Opaque)
-	if err != nil {
-		return fmt.Errorf("Could not parse to: %v", err)
+	if err != nil && addr.Opaque != "" {
+		return fmt.Errorf("Could not parse to: %w", err)
 	}
 	h.SetAddressList("to", to)
 	for key, vals := range addr.Query() {
 		switch strings.ToLower(key) {
+		case "account":
+			acctName = strings.Join(vals, "")
+		case "bcc":
+			list, err := mail.ParseAddressList(strings.Join(vals, ","))
+			if err != nil {
+				break
+			}
+			h.SetAddressList("Bcc", list)
+		case "body":
+			body = strings.Join(vals, "\n")
 		case "cc":
 			list, err := mail.ParseAddressList(strings.Join(vals, ","))
 			if err != nil {
@@ -535,17 +616,32 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		}
 	}
 
+	acct := aerc.SelectedAccount()
+	if acctName != "" {
+		if a, ok := aerc.accounts[acctName]; ok && a != nil {
+			acct = a
+		}
+	}
+
+	if acct == nil {
+		return errors.New("No account selected")
+	}
+
 	composer, err := NewComposer(aerc, acct, aerc.Config(),
 		acct.AccountConfig(), acct.Worker(), "", h, models.OriginalMail{})
 	if err != nil {
 		return nil
 	}
-	composer.FocusSubject()
+	composer.SetContents(strings.NewReader(body))
+	composer.FocusEditor("subject")
 	title := "New email"
 	if subject != "" {
 		title = subject
 		composer.FocusTerminal()
 	}
+	if to == nil {
+		composer.FocusEditor("to")
+	}
 	tab := aerc.NewTab(composer, title)
 	composer.OnHeaderChange("Subject", func(subject string) {
 		if subject == "" {
@@ -558,6 +654,33 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 	return nil
 }
 
+func (aerc *Aerc) Mbox(source string) error {
+	acctConf := config.AccountConfig{}
+	if selectedAcct := aerc.SelectedAccount(); selectedAcct != nil {
+		acctConf = *selectedAcct.acct
+		info := fmt.Sprintf("Loading outgoing mbox mail settings from account [%s]", selectedAcct.Name())
+		aerc.PushStatus(info, 10*time.Second)
+		logging.Infof(info)
+	} else {
+		acctConf.From = "<user@localhost>"
+	}
+	acctConf.Name = "mbox"
+	acctConf.Source = source
+	acctConf.Default = "INBOX"
+	acctConf.Archive = "Archive"
+	acctConf.Postpone = "Drafts"
+	acctConf.CopyTo = "Sent"
+
+	mboxView, err := NewAccountView(aerc, aerc.conf, &acctConf, aerc, nil)
+	if err != nil {
+		aerc.NewTab(errorScreen(err.Error(), aerc.conf.Ui), acctConf.Name)
+	} else {
+		aerc.accounts[acctConf.Name] = mboxView
+		aerc.NewTab(mboxView, acctConf.Name)
+	}
+	return nil
+}
+
 func (aerc *Aerc) CloseBackends() error {
 	var returnErr error
 	for _, acct := range aerc.accounts {
@@ -569,8 +692,7 @@ func (aerc *Aerc) CloseBackends() error {
 		err := c.Close()
 		if err != nil {
 			returnErr = err
-			aerc.logger.Printf("Closing backend failed for %v: %v\n",
-				acct.Name(), err)
+			logging.Errorf("Closing backend failed for %s: %v", acct.Name(), err)
 		}
 	}
 	return returnErr
@@ -582,13 +704,11 @@ func (aerc *Aerc) AddDialog(d ui.DrawableInteractive) {
 		aerc.Invalidate()
 	})
 	aerc.Invalidate()
-	return
 }
 
 func (aerc *Aerc) CloseDialog() {
 	aerc.dialog = nil
 	aerc.Invalidate()
-	return
 }
 
 func (aerc *Aerc) GetPassword(title string, prompt string) (chText chan string, chErr chan error) {
@@ -606,7 +726,6 @@ func (aerc *Aerc) GetPassword(title string, prompt string) (chText chan string,
 		}
 		chErr <- nil
 		chText <- pw
-		return
 	})
 	aerc.AddDialog(getPasswd)
 
@@ -646,11 +765,11 @@ func errorScreen(s string, conf config.UIConfig) ui.Drawable {
 	errstyle := conf.GetStyle(config.STYLE_ERROR)
 	text := ui.NewText(s, errstyle).Strategy(ui.TEXT_CENTER)
 	grid := ui.NewGrid().Rows([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
-		{ui.SIZE_EXACT, ui.Const(1)},
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
-		{ui.SIZE_WEIGHT, ui.Const(1)},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 	grid.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(0, 0)
 	grid.AddChild(text).At(1, 0)
