@@ -82,8 +82,7 @@ func init() {
 	keyMap[tcell.KeyPgUp] = directKey(vterm.KeyPageUp)
 	keyMap[tcell.KeyPgDn] = directKey(vterm.KeyPageDown)
 	for i := 0; i < 64; i++ {
-		keyMap[tcell.Key(int(tcell.KeyF1)+i)] =
-			directKey(vterm.Key(int(vterm.KeyFunction0) + i + 1))
+		keyMap[tcell.Key(int(tcell.KeyF1)+i)] = directKey(vterm.Key(int(vterm.KeyFunction0) + i + 1))
 	}
 	keyMap[tcell.KeyTAB] = directKey(vterm.KeyTab)
 	keyMap[tcell.KeyESC] = directKey(vterm.KeyEscape)
@@ -191,8 +190,14 @@ func (term *Terminal) Close(err error) {
 		term.pty = nil
 	}
 	if term.cmd != nil && term.cmd.Process != nil {
-		term.cmd.Process.Kill()
-		term.cmd.Wait()
+		err := term.cmd.Process.Kill()
+		if err != nil {
+			logging.Warnf("failed to kill process: %v", err)
+		}
+		err = term.cmd.Wait()
+		if err != nil {
+			logging.Warnf("failed for wait for process to terminate: %v", err)
+		}
 		term.cmd = nil
 	}
 	if !term.closed && term.OnClose != nil {
@@ -278,7 +283,10 @@ func (term *Terminal) Draw(ctx *ui.Context) {
 
 		if ctx.Width() != cols || ctx.Height() != rows {
 			term.writeMutex.Lock()
-			pty.Setsize(term.pty, &winsize)
+			err := pty.Setsize(term.pty, &winsize)
+			if err != nil {
+				logging.Warnf("failed to set terminal size: %v", err)
+			}
 			term.vterm.SetSize(ctx.Height(), ctx.Width())
 			term.writeMutex.Unlock()
 			rect := vterm.NewRect(0, ctx.Width(), 0, ctx.Height())
@@ -302,7 +310,6 @@ func (term *Terminal) Draw(ctx *ui.Context) {
 	term.damageMutex.Lock()
 	for _, rect := range term.damage {
 		for x := rect.StartCol(); x < rect.EndCol() && x < ctx.Width(); x += 1 {
-
 			for y := rect.StartRow(); y < rect.EndRow() && y < ctx.Height(); y += 1 {
 
 				coords := coords{x, y}
@@ -336,8 +343,7 @@ func (term *Terminal) Draw(ctx *ui.Context) {
 }
 
 func (term *Terminal) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
+	if event, ok := event.(*tcell.EventMouse); ok {
 		if term.OnEvent != nil {
 			if term.OnEvent(event) {
 				return
@@ -392,22 +398,20 @@ func (term *Terminal) Event(event tcell.Event) bool {
 	if term.closed {
 		return false
 	}
-	switch event := event.(type) {
-	case *tcell.EventKey:
+	if event, ok := event.(*tcell.EventKey); ok {
 		if event.Key() == tcell.KeyRune {
 			term.vterm.KeyboardUnichar(
 				event.Rune(), convertMods(event.Modifiers()))
-		} else {
-			if key, ok := keyMap[event.Key()]; ok {
-				if key.Key == vterm.KeyNone {
-					term.vterm.KeyboardUnichar(
-						key.Rune, key.Mod)
-				} else if key.Mod == vterm.ModNone {
-					term.vterm.KeyboardKey(key.Key,
-						convertMods(event.Modifiers()))
-				} else {
-					term.vterm.KeyboardKey(key.Key, key.Mod)
-				}
+		} else if key, ok := keyMap[event.Key()]; ok {
+			switch {
+			case key.Key == vterm.KeyNone:
+				term.vterm.KeyboardUnichar(
+					key.Rune, key.Mod)
+			case key.Mod == vterm.ModNone:
+				term.vterm.KeyboardKey(key.Key,
+					convertMods(event.Modifiers()))
+			default:
+				term.vterm.KeyboardKey(key.Key, key.Mod)
 			}
 		}
 		term.flushTerminal()
@@ -425,19 +429,21 @@ func (term *Terminal) styleFromCell(cell *vterm.ScreenCell) tcell.Style {
 		bg tcell.Color
 		fg tcell.Color
 	)
-	if background.IsDefaultBg() {
+	switch {
+	case background.IsDefaultBg():
 		bg = tcell.ColorDefault
-	} else if background.IsIndexed() {
+	case background.IsIndexed():
 		bg = tcell.Color(tcell.PaletteColor(int(background.GetIndex())))
-	} else if background.IsRgb() {
+	case background.IsRgb():
 		r, g, b := background.GetRGB()
 		bg = tcell.NewRGBColor(int32(r), int32(g), int32(b))
 	}
-	if foreground.IsDefaultFg() {
+	switch {
+	case foreground.IsDefaultFg():
 		fg = tcell.ColorDefault
-	} else if foreground.IsIndexed() {
+	case foreground.IsIndexed():
 		fg = tcell.Color(tcell.PaletteColor(int(foreground.GetIndex())))
-	} else if foreground.IsRgb() {
+	case foreground.IsRgb():
 		r, g, b := foreground.GetRGB()
 		fg = tcell.NewRGBColor(int32(r), int32(g), int32(b))
 	}
@@ -472,8 +478,8 @@ func (term *Terminal) onDamage(rect *vterm.Rect) int {
 }
 
 func (term *Terminal) onMoveCursor(old *vterm.Pos,
-	pos *vterm.Pos, visible bool) int {
-
+	pos *vterm.Pos, visible bool,
+) int {
 	rows, cols, _ := pty.Getsize(term.pty)
 	if pos.Row() >= rows || pos.Col() >= cols {
 		return 1
