@@ -2,27 +2,32 @@ package maildir
 
 import (
 	"fmt"
-	"log"
 	"os"
 	"path/filepath"
+	"regexp"
 	"sort"
+	"strings"
 
 	"github.com/emersion/go-maildir"
 
 	"git.sr.ht/~rjarry/aerc/lib/uidstore"
 )
 
+// uidReg matches filename encoded UIDs in maildirs synched with mbsync or
+// OfflineIMAP
+var uidReg = regexp.MustCompile(`,U=\d+`)
+
 // A Container is a directory which contains other directories which adhere to
 // the Maildir spec
 type Container struct {
 	dir        string
-	log        *log.Logger
 	uids       *uidstore.Store
 	recentUIDS map[uint32]struct{} // used to set the recent flag
+	maildirpp  bool                // whether to use Maildir++ directory layout
 }
 
 // NewContainer creates a new container at the specified directory
-func NewContainer(dir string, l *log.Logger) (*Container, error) {
+func NewContainer(dir string, maildirpp bool) (*Container, error) {
 	f, err := os.Open(dir)
 	if err != nil {
 		return nil, err
@@ -35,17 +40,22 @@ func NewContainer(dir string, l *log.Logger) (*Container, error) {
 	if !s.IsDir() {
 		return nil, fmt.Errorf("Given maildir '%s' not a directory", dir)
 	}
-	return &Container{dir: dir, uids: uidstore.NewStore(), log: l,
-		recentUIDS: make(map[uint32]struct{})}, nil
+	return &Container{
+		dir: dir, uids: uidstore.NewStore(),
+		recentUIDS: make(map[uint32]struct{}), maildirpp: maildirpp,
+	}, nil
 }
 
 // ListFolders returns a list of maildir folders in the container
 func (c *Container) ListFolders() ([]string, error) {
 	folders := []string{}
+	if c.maildirpp {
+		// In Maildir++ layout, INBOX is the root folder
+		folders = append(folders, "INBOX")
+	}
 	err := filepath.Walk(c.dir, func(path string, info os.FileInfo, err error) error {
 		if err != nil {
-			return fmt.Errorf("Invalid path '%s': error: %v", path, err)
-
+			return fmt.Errorf("Invalid path '%s': error: %w", path, err)
 		}
 		if !info.IsDir() {
 			return nil
@@ -68,6 +78,21 @@ func (c *Container) ListFolders() ([]string, error) {
 			return nil
 		}
 
+		if c.maildirpp {
+			// In Maildir++ layout, mailboxes are stored in a single directory
+			// and prefixed with a dot, and subfolders are separated by dots.
+			if !strings.HasPrefix(dirPath, ".") {
+				return filepath.SkipDir
+			}
+			dirPath = strings.TrimPrefix(dirPath, ".")
+			dirPath = strings.ReplaceAll(dirPath, ".", "/")
+			folders = append(folders, dirPath)
+
+			// Since all mailboxes are stored in a single directory, don't
+			// recurse into subdirectories
+			return filepath.SkipDir
+		}
+
 		folders = append(folders, dirPath)
 		return nil
 	})
@@ -99,6 +124,13 @@ func (c *Container) OpenDirectory(name string) (maildir.Dir, error) {
 
 // Dir returns a maildir.Dir with the specified name inside the container
 func (c *Container) Dir(name string) maildir.Dir {
+	if c.maildirpp {
+		// Use Maildir++ layout
+		if name == "INBOX" {
+			return maildir.Dir(c.dir)
+		}
+		return maildir.Dir(filepath.Join(c.dir, "."+strings.ReplaceAll(name, "/", ".")))
+	}
 	return maildir.Dir(filepath.Join(c.dir, name))
 }
 
@@ -117,7 +149,7 @@ func (c *Container) ClearRecentFlag(uid uint32) {
 func (c *Container) UIDs(d maildir.Dir) ([]uint32, error) {
 	keys, err := d.Keys()
 	if err != nil {
-		return nil, fmt.Errorf("could not get keys for %s: %v", d, err)
+		return nil, fmt.Errorf("could not get keys for %s: %w", d, err)
 	}
 	sort.Strings(keys)
 	var uids []uint32
@@ -158,17 +190,19 @@ func (c *Container) DeleteAll(d maildir.Dir, uids []uint32) ([]uint32, error) {
 }
 
 func (c *Container) CopyAll(
-	dest maildir.Dir, src maildir.Dir, uids []uint32) error {
+	dest maildir.Dir, src maildir.Dir, uids []uint32,
+) error {
 	for _, uid := range uids {
 		if err := c.copyMessage(dest, src, uid); err != nil {
-			return fmt.Errorf("could not copy message %d: %v", uid, err)
+			return fmt.Errorf("could not copy message %d: %w", uid, err)
 		}
 	}
 	return nil
 }
 
 func (c *Container) copyMessage(
-	dest maildir.Dir, src maildir.Dir, uid uint32) error {
+	dest maildir.Dir, src maildir.Dir, uid uint32,
+) error {
 	key, ok := c.uids.GetKey(uid)
 	if !ok {
 		return fmt.Errorf("could not find key for message id %d", uid)
@@ -176,3 +210,29 @@ func (c *Container) copyMessage(
 	_, err := src.Copy(dest, key)
 	return err
 }
+
+func (c *Container) MoveAll(dest maildir.Dir, src maildir.Dir, uids []uint32) ([]uint32, error) {
+	var success []uint32
+	for _, uid := range uids {
+		if err := c.moveMessage(dest, src, uid); err != nil {
+			return success, fmt.Errorf("could not move message %d: %w", uid, err)
+		}
+		success = append(success, uid)
+	}
+	return success, nil
+}
+
+func (c *Container) moveMessage(dest maildir.Dir, src maildir.Dir, uid uint32) error {
+	key, ok := c.uids.GetKey(uid)
+	if !ok {
+		return fmt.Errorf("could not find key for message id %d", uid)
+	}
+	path, err := src.Filename(key)
+	if err != nil {
+		return fmt.Errorf("could not find path for message id %d", uid)
+	}
+	// Remove encoded UID information from the key to prevent sync issues
+	name := uidReg.ReplaceAllString(filepath.Base(path), "")
+	destPath := filepath.Join(string(dest), "cur", name)
+	return os.Rename(path, destPath)
+}
