@@ -6,7 +6,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/url"
 	"os"
 	"os/exec"
@@ -17,6 +16,7 @@ import (
 	"github.com/emersion/go-maildir"
 	"github.com/fsnotify/fsnotify"
 
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -45,7 +45,7 @@ type Worker struct {
 func NewWorker(worker *types.Worker) (types.Backend, error) {
 	watch, err := fsnotify.NewWatcher()
 	if err != nil {
-		return nil, fmt.Errorf("could not create file system watcher: %v", err)
+		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
 	return &Worker{worker: worker, watcher: watch}, nil
 }
@@ -54,7 +54,7 @@ func NewWorker(worker *types.Worker) (types.Backend, error) {
 func NewMaildirppWorker(worker *types.Worker) (types.Backend, error) {
 	watch, err := fsnotify.NewWatcher()
 	if err != nil {
-		return nil, fmt.Errorf("could not create file system watcher: %v", err)
+		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
 	return &Worker{worker: worker, watcher: watch, maildirpp: true}, nil
 }
@@ -80,16 +80,18 @@ func (w *Worker) handleAction(action types.WorkerMessage) {
 		go w.handleCheckMail(msg)
 	default:
 		// Default handling, will be performed synchronously
-		if err := w.handleMessage(msg); err == errUnsupported {
+		err := w.handleMessage(msg)
+		switch {
+		case errors.Is(err, errUnsupported):
 			w.worker.PostMessage(&types.Unsupported{
 				Message: types.RespondTo(msg),
 			}, nil)
-		} else if err != nil {
+		case err != nil:
 			w.worker.PostMessage(&types.Error{
 				Message: types.RespondTo(msg),
 				Error:   err,
 			}, nil)
-		} else {
+		default:
 			w.done(msg)
 		}
 	}
@@ -109,23 +111,10 @@ func (w *Worker) handleFSEvent(ev fsnotify.Event) {
 	}
 	err := w.c.SyncNewMail(*w.selected)
 	if err != nil {
-		w.worker.Logger.Printf("could not move new to cur : %v", err)
+		logging.Errorf("could not move new to cur : %w", err)
 		return
 	}
 
-	uids, err := w.c.UIDs(*w.selected)
-	if err != nil {
-		w.worker.Logger.Printf("could not scan UIDs: %v", err)
-		return
-	}
-	sortedUids, err := w.sort(uids, w.currentSortCriteria)
-	if err != nil {
-		w.worker.Logger.Printf("error sorting directory: %v", err)
-		return
-	}
-	w.worker.PostMessage(&types.DirectoryContents{
-		Uids: sortedUids,
-	}, nil)
 	dirInfo := w.getDirectoryInfo(w.selectedName)
 	w.worker.PostMessage(&types.DirectoryInfo{
 		Info: dirInfo,
@@ -201,18 +190,18 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 		for _, v := range files {
 			key, flags, err := splitMaildirFile(v)
 			if err != nil {
-				w.worker.Logger.Printf("%q: error parsing flags (%q): %v", v, key, err)
+				logging.Errorf("%q: error parsing flags (%q): %w", v, key, err)
 				continue
 			}
 			keyFlags[key] = flags
 		}
 	} else {
-		w.worker.Logger.Printf("disabled flags cache: %q: %v", dir, err)
+		logging.Infof("disabled flags cache: %q: %w", dir, err)
 	}
 
 	uids, err := w.c.UIDs(dir)
 	if err != nil {
-		w.worker.Logger.Printf("could not get uids: %v", err)
+		logging.Errorf("could not get uids: %w", err)
 		return dirInfo
 	}
 
@@ -220,7 +209,7 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 	for _, uid := range uids {
 		message, err := w.c.Message(dir, uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			continue
 		}
 		var flags []maildir.Flag
@@ -228,17 +217,17 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 			ok := false
 			flags, ok = keyFlags[message.key]
 			if !ok {
-				w.worker.Logger.Printf("message (key=%q uid=%d) not found in map cache", message.key, message.uid)
+				logging.Debugf("message (key=%q uid=%d) not found in map cache", message.key, message.uid)
 				flags, err = message.Flags()
 				if err != nil {
-					w.worker.Logger.Printf("could not get flags: %v", err)
+					logging.Errorf("could not get flags: %w", err)
 					continue
 				}
 			}
 		} else {
 			flags, err = message.Flags()
 			if err != nil {
-				w.worker.Logger.Printf("could not get flags: %v", err)
+				logging.Errorf("could not get flags: %w", err)
 				continue
 			}
 		}
@@ -292,6 +281,8 @@ func (w *Worker) handleMessage(msg types.WorkerMessage) error {
 		return w.handleAnsweredMessages(msg)
 	case *types.CopyMessages:
 		return w.handleCopyMessages(msg)
+	case *types.MoveMessages:
+		return w.handleMoveMessages(msg)
 	case *types.AppendMessage:
 		return w.handleAppendMessage(msg)
 	case *types.SearchDirectory:
@@ -303,27 +294,27 @@ func (w *Worker) handleMessage(msg types.WorkerMessage) error {
 func (w *Worker) handleConfigure(msg *types.Configure) error {
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		w.worker.Logger.Printf("error configuring maildir worker: %v", err)
+		logging.Errorf("error configuring maildir worker: %w", err)
 		return err
 	}
 	dir := u.Path
 	if u.Host == "~" {
 		home, err := os.UserHomeDir()
 		if err != nil {
-			return fmt.Errorf("could not resolve home directory: %v", err)
+			return fmt.Errorf("could not resolve home directory: %w", err)
 		}
 		dir = filepath.Join(home, u.Path)
 	}
 	if len(dir) == 0 {
 		return fmt.Errorf("could not resolve maildir from URL '%s'", msg.Config.Source)
 	}
-	c, err := NewContainer(dir, w.worker.Logger, w.maildirpp)
+	c, err := NewContainer(dir, w.maildirpp)
 	if err != nil {
-		w.worker.Logger.Printf("could not configure maildir: %s", dir)
+		logging.Errorf("could not configure maildir: %s", dir)
 		return err
 	}
 	w.c = c
-	w.worker.Logger.Printf("configured base maildir: %s", dir)
+	logging.Infof("configured base maildir: %s", dir)
 	return nil
 }
 
@@ -340,7 +331,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 	}
 	dirs, err := w.c.ListFolders()
 	if err != nil {
-		w.worker.Logger.Printf("error listing directories: %v", err)
+		logging.Errorf("failed listing directories: %w", err)
 		return err
 	}
 	for _, name := range dirs {
@@ -360,7 +351,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 }
 
 func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	w.worker.Logger.Printf("opening %s", msg.Directory)
+	logging.Infof("opening %s", msg.Directory)
 
 	// open the directory
 	dir, err := w.c.OpenDirectory(msg.Directory)
@@ -372,11 +363,11 @@ func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 	if w.selected != nil {
 		prevDir := filepath.Join(string(*w.selected), "new")
 		if err := w.watcher.Remove(prevDir); err != nil {
-			return fmt.Errorf("could not unwatch previous directory: %v", err)
+			return fmt.Errorf("could not unwatch previous directory: %w", err)
 		}
 		prevDir = filepath.Join(string(*w.selected), "cur")
 		if err := w.watcher.Remove(prevDir); err != nil {
-			return fmt.Errorf("could not unwatch previous directory: %v", err)
+			return fmt.Errorf("could not unwatch previous directory: %w", err)
 		}
 	}
 
@@ -386,15 +377,15 @@ func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 	// add watch paths
 	newDir := filepath.Join(string(*w.selected), "new")
 	if err := w.watcher.Add(newDir); err != nil {
-		return fmt.Errorf("could not add watch to directory: %v", err)
+		return fmt.Errorf("could not add watch to directory: %w", err)
 	}
 	newDir = filepath.Join(string(*w.selected), "cur")
 	if err := w.watcher.Add(newDir); err != nil {
-		return fmt.Errorf("could not add watch to directory: %v", err)
+		return fmt.Errorf("could not add watch to directory: %w", err)
 	}
 
 	if err := dir.Clean(); err != nil {
-		return fmt.Errorf("could not clean directory: %v", err)
+		return fmt.Errorf("could not clean directory: %w", err)
 	}
 
 	info := &types.DirectoryInfo{
@@ -405,7 +396,8 @@ func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 }
 
 func (w *Worker) handleFetchDirectoryContents(
-	msg *types.FetchDirectoryContents) error {
+	msg *types.FetchDirectoryContents,
+) error {
 	var (
 		uids []uint32
 		err  error
@@ -422,13 +414,13 @@ func (w *Worker) handleFetchDirectoryContents(
 	} else {
 		uids, err = w.c.UIDs(*w.selected)
 		if err != nil {
-			w.worker.Logger.Printf("error scanning uids: %v", err)
+			logging.Errorf("failed scanning uids: %w", err)
 			return err
 		}
 	}
 	sortedUids, err := w.sort(uids, msg.SortCriteria)
 	if err != nil {
-		w.worker.Logger.Printf("error sorting directory: %v", err)
+		logging.Errorf("failed sorting directory: %w", err)
 		return err
 	}
 	w.currentSortCriteria = msg.SortCriteria
@@ -447,14 +439,14 @@ func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32,
 	for _, uid := range uids {
 		info, err := w.msgInfoFromUid(uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message info: %v", err)
+			logging.Errorf("could not get message info: %w", err)
 			continue
 		}
 		msgInfos = append(msgInfos, info)
 	}
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		w.worker.Logger.Printf("could not sort the messages: %v", err)
+		logging.Errorf("could not sort the messages: %w", err)
 		return nil, err
 	}
 	return sortedUids, nil
@@ -463,7 +455,7 @@ func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32,
 func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 	dir := w.c.Dir(msg.Directory)
 	if err := dir.Init(); err != nil {
-		w.worker.Logger.Printf("could not create directory %s: %v",
+		logging.Errorf("could not create directory %s: %w",
 			msg.Directory, err)
 		return err
 	}
@@ -473,7 +465,7 @@ func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 func (w *Worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
 	dir := w.c.Dir(msg.Directory)
 	if err := os.RemoveAll(string(dir)); err != nil {
-		w.worker.Logger.Printf("could not remove directory %s: %v",
+		logging.Errorf("could not remove directory %s: %w",
 			msg.Directory, err)
 		return err
 	}
@@ -481,11 +473,12 @@ func (w *Worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
 }
 
 func (w *Worker) handleFetchMessageHeaders(
-	msg *types.FetchMessageHeaders) error {
+	msg *types.FetchMessageHeaders,
+) error {
 	for _, uid := range msg.Uids {
 		info, err := w.msgInfoFromUid(uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message info: %v", err)
+			logging.Errorf("could not get message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
@@ -499,18 +492,18 @@ func (w *Worker) handleFetchMessageHeaders(
 }
 
 func (w *Worker) handleFetchMessageBodyPart(
-	msg *types.FetchMessageBodyPart) error {
-
+	msg *types.FetchMessageBodyPart,
+) error {
 	// get reader
 	m, err := w.c.Message(*w.selected, msg.Uid)
 	if err != nil {
-		w.worker.Logger.Printf("could not get message %d: %v", msg.Uid, err)
+		logging.Errorf("could not get message %d: %w", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		w.worker.Logger.Printf(
-			"could not get body part reader for message=%d, parts=%#v: %v",
+		logging.Errorf(
+			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
 	}
@@ -529,16 +522,16 @@ func (w *Worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message %d: %v", uid, err)
+			logging.Errorf("could not get message %d: %w", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			w.worker.Logger.Printf("could not get message reader: %v", err)
+			logging.Errorf("could not get message reader: %w", err)
 			return err
 		}
 		defer r.Close()
-		b, err := ioutil.ReadAll(r)
+		b, err := io.ReadAll(r)
 		if err != nil {
 			return err
 		}
@@ -550,6 +543,9 @@ func (w *Worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 			},
 		}, nil)
 	}
+	w.worker.PostMessage(&types.Done{
+		Message: types.RespondTo(msg),
+	}, nil)
 	return nil
 }
 
@@ -562,7 +558,7 @@ func (w *Worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 		}, nil)
 	}
 	if err != nil {
-		w.worker.Logger.Printf("error removing some messages: %v", err)
+		logging.Errorf("failed removing messages: %w", err)
 		return err
 	}
 	return nil
@@ -572,19 +568,18 @@ func (w *Worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkReplied(msg.Answered); err != nil {
-			w.worker.Logger.Printf(
-				"could not mark message as answered: %v", err)
+			logging.Errorf("could not mark message as answered: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			w.worker.Logger.Printf("could not get message info: %v", err)
+			logging.Errorf("could not get message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
@@ -605,19 +600,19 @@ func (w *Worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			w.worker.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		flag := flagToMaildir[msg.Flag]
 		if err := m.SetOneFlag(flag, msg.Enable); err != nil {
-			w.worker.Logger.Printf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
+			logging.Errorf("could change flag %v to %v on message: %w", flag, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			w.worker.Logger.Printf("could not get message info: %v", err)
+			logging.Errorf("could not get message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
@@ -649,21 +644,36 @@ func (w *Worker) handleCopyMessages(msg *types.CopyMessages) error {
 	return nil
 }
 
+func (w *Worker) handleMoveMessages(msg *types.MoveMessages) error {
+	dest := w.c.Dir(msg.Destination)
+	moved, err := w.c.MoveAll(dest, *w.selected, msg.Uids)
+	destInfo := w.getDirectoryInfo(msg.Destination)
+	w.worker.PostMessage(&types.DirectoryInfo{
+		Info: destInfo,
+	}, nil)
+	w.worker.PostMessage(&types.MessagesDeleted{
+		Message: types.RespondTo(msg),
+		Uids:    moved,
+	}, nil)
+	return err
+}
+
 func (w *Worker) handleAppendMessage(msg *types.AppendMessage) error {
 	// since we are the "master" maildir process, we can modify the maildir directly
 	dest := w.c.Dir(msg.Destination)
 	_, writer, err := dest.Create(translateFlags(msg.Flags))
 	if err != nil {
-		w.worker.Logger.Printf("could not create message at %s: %v",
-			msg.Destination, err)
+		logging.Errorf("could not create message at %s: %w", msg.Destination, err)
 		return err
 	}
 	defer writer.Close()
 	if _, err := io.Copy(writer, msg.Reader); err != nil {
-		w.worker.Logger.Printf("could not write message to destination: %v", err)
+		logging.Errorf("could not write message to destination: %w", err)
 		return err
 	}
-
+	w.worker.PostMessage(&types.Done{
+		Message: types.RespondTo(msg),
+	}, nil)
 	w.worker.PostMessage(&types.DirectoryInfo{
 		Info: w.getDirectoryInfo(msg.Destination),
 	}, nil)
@@ -671,12 +681,12 @@ func (w *Worker) handleAppendMessage(msg *types.AppendMessage) error {
 }
 
 func (w *Worker) handleSearchDirectory(msg *types.SearchDirectory) error {
-	w.worker.Logger.Printf("Searching directory %v with args: %v", *w.selected, msg.Argv)
+	logging.Infof("Searching directory %v with args: %v", *w.selected, msg.Argv)
 	criteria, err := parseSearch(msg.Argv)
 	if err != nil {
 		return err
 	}
-	w.worker.Logger.Printf("Searching with parsed criteria: %#v", criteria)
+	logging.Infof("Searching with parsed criteria: %#v", criteria)
 	uids, err := w.search(criteria)
 	if err != nil {
 		return err
@@ -704,6 +714,10 @@ func (w *Worker) msgInfoFromUid(uid uint32) (*models.MessageInfo, error) {
 }
 
 func (w *Worker) handleCheckMail(msg *types.CheckMail) {
+	if msg.Command == "" {
+		w.err(msg, fmt.Errorf("checkmail: no command specified"))
+		return
+	}
 	ctx, cancel := context.WithTimeout(context.Background(), msg.Timeout)
 	defer cancel()
 	cmd := exec.CommandContext(ctx, "sh", "-c", msg.Command)
@@ -717,7 +731,7 @@ func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 		w.err(msg, fmt.Errorf("checkmail: timed out"))
 	case err := <-ch:
 		if err != nil {
-			w.err(msg, fmt.Errorf("checkmail: error running command: %v", err))
+			w.err(msg, fmt.Errorf("checkmail: error running command: %w", err))
 		} else {
 			w.done(msg)
 		}
